@@ -45,11 +45,11 @@ private:
 
 BasicTaskScheduler0::BasicTaskScheduler0()
   : fLastHandledSocketNum(-1) {
-  fReadHandlers = new HandlerSet;
+  fHandlers = new HandlerSet;
 }
 
 BasicTaskScheduler0::~BasicTaskScheduler0() {
-  delete fReadHandlers;
+  delete fHandlers;
 }
 
 TaskToken BasicTaskScheduler0::scheduleDelayedTask(int64_t microseconds,
@@ -80,7 +80,8 @@ void BasicTaskScheduler0::doEventLoop(char* watchVariable) {
 
 ////////// HandlerSet (etc.) implementation //////////
 
-HandlerDescriptor::HandlerDescriptor(HandlerDescriptor* nextHandler) {
+HandlerDescriptor::HandlerDescriptor(HandlerDescriptor* nextHandler)
+  : conditionSet(0), handlerProc(NULL) {
   // Link this descriptor into a doubly-linked list:
   if (nextHandler == this) { // initialization
     fNextHandler = fPrevHandler = this;
@@ -111,33 +112,38 @@ HandlerSet::~HandlerSet() {
 }
 
 void HandlerSet
-::assignHandler(int socketNum,
-		TaskScheduler::BackgroundHandlerProc* handlerProc,
-		void* clientData) {
+::assignHandler(int socketNum, int conditionSet, TaskScheduler::BackgroundHandlerProc* handlerProc, void* clientData) {
   // First, see if there's already a handler for this socket:
-  HandlerDescriptor* handler;
-  HandlerIterator iter(*this);
-  while ((handler = iter.next()) != NULL) {
-    if (handler->socketNum == socketNum) break;
-  }
+  HandlerDescriptor* handler = lookupHandler(socketNum);
   if (handler == NULL) { // No existing handler, so create a new descr:
     handler = new HandlerDescriptor(fHandlers.fNextHandler);
     handler->socketNum = socketNum;
   }
 
+  handler->conditionSet = conditionSet;
   handler->handlerProc = handlerProc;
   handler->clientData = clientData;
 }
 
-void HandlerSet::removeHandler(int socketNum) {
+void HandlerSet::clearHandler(int socketNum) {
+  HandlerDescriptor* handler = lookupHandler(socketNum);
+  delete handler;
+}
+
+void HandlerSet::moveHandler(int oldSocketNum, int newSocketNum) {
+  HandlerDescriptor* handler = lookupHandler(oldSocketNum);
+  if (handler != NULL) {
+    handler->socketNum = newSocketNum;
+  }
+}
+
+HandlerDescriptor* HandlerSet::lookupHandler(int socketNum) {
   HandlerDescriptor* handler;
   HandlerIterator iter(*this);
   while ((handler = iter.next()) != NULL) {
-    if (handler->socketNum == socketNum) {
-      delete handler;
-      break;
-    }
+    if (handler->socketNum == socketNum) break;
   }
+  return handler;
 }
 
 HandlerIterator::HandlerIterator(HandlerSet& handlerSet)
