@@ -11,10 +11,10 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2010 Live Networks, Inc.  All rights reserved.
 // A source that consists of multiple byte-stream files, read sequentially
 // Implementation
 
@@ -86,7 +86,7 @@ void ByteStreamMultiFileSource::doGetNextFrame() {
 		       fPreferredFrameSize, fPlayTimePerFrame);
       if (source == NULL) break;
       fHaveStartedNewFile = True;
-    }      
+    }
 
     // (Attempt to) read from the current source.
     source->getNextFrame(fTo, fMaxSize,
