@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2008 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2010 Live Networks, Inc.  All rights reserved.
 // A class used for digest authentication.
 // Implementation
 
@@ -30,6 +30,10 @@ Authenticator::Authenticator() {
   assign(NULL, NULL, NULL, NULL, False);
 }
 
+Authenticator::Authenticator(char const* username, char const* password) {
+  setUsernameAndPassword(username, password);
+}
+
 Authenticator::Authenticator(const Authenticator& orig) {
   assign(orig.realm(), orig.nonce(), orig.username(), orig.password(),
 	 orig.fPasswordIsMD5);
