@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2008 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2010 Live Networks, Inc.  All rights reserved.
 // JPEG video sources
 // Implementation
 
@@ -35,6 +35,11 @@ u_int8_t const* JPEGVideoSource::quantizationTables(u_int8_t& precision,
   return NULL;
 }
 
+u_int16_t JPEGVideoSource::restartInterval() {
+  // Default implementation
+  return 0;
+}
+
 Boolean JPEGVideoSource::isJPEGVideoSource() const {
   return True;
 }
