@@ -82,6 +82,7 @@ private:
 private:
   // Redefined virtual functions:
   void doGetNextFrame();
+  virtual void doStopGettingFrames();
 
 private:
   class QCELPDeinterleavingBuffer* fDeinterleavingBuffer;
@@ -349,6 +350,11 @@ void QCELPDeinterleaver::doGetNextFrame() {
   }
 }
 
+void QCELPDeinterleaver::doStopGettingFrames() {
+  fNeedAFrame = False;
+  fInputSource->stopGettingFrames();
+}
+
 void QCELPDeinterleaver
 ::afterGettingFrame(void* clientData, unsigned frameSize,
 		    unsigned /*numTruncatedBytes*/,
