@@ -1140,7 +1140,7 @@ Boolean RTSPClient::handleAuthenticationFailure(char const* paramsStr) {
 
 Boolean RTSPClient::resendCommand(RequestRecord* request) {
   if (fVerbosityLevel >= 1) envir() << "Resending...\n";
-  if (request != NULL) request->cseq() = ++fCSeq;
+  if (request != NULL && strcmp(request->commandName(), "GET") != 0) request->cseq() = ++fCSeq;
   return sendRequest(request) != 0;
 }
 
@@ -1202,7 +1202,7 @@ void RTSPClient::constructSubsessionURL(MediaSubsession const& subsession,
 
 Boolean RTSPClient::setupHTTPTunneling1() {
   // Set up RTSP-over-HTTP tunneling, as described in
-  //     http://developer.apple.com/documentation/QuickTime/QTSS/Concepts/chapter_2_section_14.html
+  //     http://developer.apple.com/quicktime/icefloe/dispatch028.html and http://images.apple.com/br/quicktime/pdf/QTSS_Modules.pdf
   if (fVerbosityLevel >= 1) {
     envir() << "Requesting RTSP-over-HTTP tunneling (on port " << fTunnelOverHTTPPortNum << ")\n\n";
   }
@@ -1532,7 +1532,15 @@ void RTSPClient::handleResponseBytes(int newBytesRead) {
 	  if (!handleGET_PARAMETERResponse(foundRequest->contentStr(), bodyStart)) break;
 	}
       } else if (responseCode == 401 && handleAuthenticationFailure(wwwAuthenticateParamsStr)) {
+	// We need to resend the command, with an "Authorization:" header:
 	needToResendCommand = True;
+
+	if (strcmp(foundRequest->commandName(), "GET") == 0) {
+	  // Note: If a HTTP "GET" command (for RTSP-over-HTTP tunneling) returns "401 Unauthorized", then we resend it
+	  // (with an "Authorization:" header), just as we would for a RTSP command.  However, we do so using a new TCP connection,
+	  // because some servers close the original connection after returning the "401 Unauthorized".
+	  resetTCPSockets(); // forces the opening of a new connection for the resent command
+	}
       } else if (responseCode == 301 || responseCode == 302) { // redirection
 	resetTCPSockets(); // because we need to connect somewhere else next
 	needToResendCommand = True;
