@@ -96,6 +96,18 @@ void RTSPServer::removeServerMediaSession(char const* streamName) {
   removeServerMediaSession(lookupServerMediaSession(streamName));
 }
 
+char* RTSPServer
+::rtspURL(ServerMediaSession const* serverMediaSession, int clientSocket) const {
+  char* urlPrefix = rtspURLPrefix(clientSocket);
+  char const* sessionName = serverMediaSession->streamName();
+
+  char* resultURL = new char[strlen(urlPrefix) + strlen(sessionName) + 1];
+  sprintf(resultURL, "%s%s", urlPrefix, sessionName);
+
+  delete[] urlPrefix;
+  return resultURL;
+}
+
 char* RTSPServer::rtspURLPrefix(int clientSocket) const {
   struct sockaddr_in ourAddress;
   if (clientSocket < 0) {
@@ -121,16 +133,11 @@ char* RTSPServer::rtspURLPrefix(int clientSocket) const {
   return strDup(urlBuffer);
 }
 
-char* RTSPServer
-::rtspURL(ServerMediaSession const* serverMediaSession, int clientSocket) const {
-  char* urlPrefix = rtspURLPrefix(clientSocket);
-  char const* sessionName = serverMediaSession->streamName();
+UserAuthenticationDatabase* RTSPServer::setAuthenticationDatabase(UserAuthenticationDatabase* newDB) {
+  UserAuthenticationDatabase* oldDB = fAuthDB;
+  fAuthDB = newDB;
 
-  char* resultURL = new char[strlen(urlPrefix) + strlen(sessionName) + 1];
-  sprintf(resultURL, "%s%s", urlPrefix, sessionName);
-
-  delete[] urlPrefix;
-  return resultURL;
+  return oldDB;
 }
 
 Boolean RTSPServer::setUpTunnelingOverHTTP(Port httpPort) {
@@ -467,27 +474,22 @@ void RTSPServer::RTSPClientSession::handleRequestBytes(int newBytesRead) {
 #endif
     // The request was not (valid) RTSP, but check for a special case: HTTP commands (for setting up RTSP-over-HTTP tunneling):
     char sessionCookie[RTSP_PARAM_STRING_MAX];
-    char acceptStr[RTSP_PARAM_STRING_MAX];
-    char contentTypeStr[RTSP_PARAM_STRING_MAX];
-    if (parseHTTPRequestString(cmdName, sizeof cmdName,
-			       sessionCookie, sizeof sessionCookie,
-			       acceptStr, sizeof acceptStr,
-			       contentTypeStr, sizeof contentTypeStr)) {
+    if (parseHTTPRequestString(cmdName, sizeof cmdName, sessionCookie, sizeof sessionCookie)) {
 #ifdef DEBUG
-      fprintf(stderr, "parseHTTPRequestString() succeeded, returning cmdName \"%s\", sessionCookie \"%s\", acceptStr \"%s\", contentTypeStr \"%s\"\n", cmdName, sessionCookie, acceptStr, contentTypeStr);
+      fprintf(stderr, "parseHTTPRequestString() succeeded, returning cmdName \"%s\", sessionCookie \"%s\"\n", cmdName, sessionCookie);
 #endif
-      // Check that the HTTP command is valid for RTSP-over-HTTP tunneling:
-      // There must be a 'session cookie', and an "Accept:" or "Content-Type:" string of "application/x-rtsp-tunnelled".
-      char const* rtspTunnelledStr = "application/x-rtsp-tunnelled";
+      // Check that the HTTP command is valid for RTSP-over-HTTP tunneling: There must be a 'session cookie'.
       Boolean isValidHTTPTunnelingCmd = True;
-      if (sessionCookie[0] == '\0' ||
-	  (strncmp(acceptStr, rtspTunnelledStr, sizeof acceptStr) != 0 &&
-	   strncmp(contentTypeStr, rtspTunnelledStr, sizeof contentTypeStr) != 0)) {
+      if (sessionCookie[0] == '\0') {
 	isValidHTTPTunnelingCmd = False;
       } else if (strcmp(cmdName, "GET") == 0) {
 	handleHTTPCmd_GET(sessionCookie);
       } else if (strcmp(cmdName, "POST") == 0) {
-	if (handleHTTPCmd_POST(sessionCookie)) {
+	// We might have received additional data following the HTTP "POST" command - i.e., the first Base64-encoded RTSP command.
+	// Check for this, and handle it if it exists:
+	unsigned char const* extraData = fLastCRLF+4;
+	unsigned extraDataSize = &fRequestBuffer[fRequestBytesAlreadySeen] - extraData;
+	if (handleHTTPCmd_POST(sessionCookie, extraData, extraDataSize)) {
 	  // We don't respond to the "POST" command, and we go away:
 	  delete this;
 	  return;
@@ -810,10 +812,13 @@ void RTSPServer::RTSPClientSession
 		       clientsDestinationAddressStr, clientsDestinationTTL,
 		       clientRTPPortNum, clientRTCPPortNum,
 		       rtpChannelId, rtcpChannelId);
-  if (streamingMode == RTP_TCP && rtpChannelId == 0xFF) {
-    // TCP streaming was requested, but with no "interleaving=" fields.
-    // (QuickTime Player sometimes does this.)  Set the RTP and RTCP channel ids to
-    // proper values:
+  if (streamingMode == RTP_TCP && rtpChannelId == 0xFF ||
+      streamingMode != RTP_TCP && fClientOutputSocket != fClientInputSocket) {
+    // An anomolous situation, caused by a buggy client.  Either:
+    //     1/ TCP streaming was requested, but with no "interleaving=" fields.  (QuickTime Player sometimes does this.), or
+    //     2/ TCP streaming was not requested, but we're doing RTSP-over-HTTP tunneling (which implies TCP streaming).
+    // In either case, we assume TCP streaming, and set the RTP and RTCP channel ids to proper values:
+    streamingMode = RTP_TCP;
     rtpChannelId = fTCPStreamIdCount; rtcpChannelId = fTCPStreamIdCount+1;
   }
   fTCPStreamIdCount += 2;
@@ -1226,9 +1231,7 @@ static void lookForHeader(char const* headerName, char const* source, unsigned s
 }
 
 Boolean RTSPServer::RTSPClientSession::parseHTTPRequestString(char* resultCmdName, unsigned resultCmdNameMaxSize,
-							   char* sessionCookie, unsigned sessionCookieMaxSize,
-							   char* acceptStr, unsigned acceptStrMaxSize,
-							   char* contentTypeStr, unsigned contentTypeStrMaxSize) {
+							      char* sessionCookie, unsigned sessionCookieMaxSize) {
   // Check for the limited HTTP requests that we expect for specifying RTSP-over-HTTP tunneling.
   // This parser is currently rather dumb; it should be made smarter #####
   char const* reqStr = (char const*)fRequestBuffer;
@@ -1262,8 +1265,6 @@ Boolean RTSPServer::RTSPClientSession::parseHTTPRequestString(char* resultCmdNam
 
   // Look for various headers that we're interested in:
   lookForHeader("x-sessioncookie", &reqStr[i], reqStrSize-i, sessionCookie, sessionCookieMaxSize);
-  lookForHeader("Accept", &reqStr[i], reqStrSize-i, acceptStr, acceptStrMaxSize);
-  lookForHeader("Content-Type", &reqStr[i], reqStrSize-i, contentTypeStr, contentTypeStrMaxSize);
 
   return True;
 }
@@ -1295,7 +1296,8 @@ void RTSPServer::RTSPClientSession::handleHTTPCmd_GET(char const* sessionCookie)
 	   "\r\n");
 }
 
-Boolean RTSPServer::RTSPClientSession::handleHTTPCmd_POST(char const* sessionCookie) {
+Boolean RTSPServer::RTSPClientSession
+::handleHTTPCmd_POST(char const* sessionCookie, unsigned char const* extraData, unsigned extraDataSize) {
   // Use the "sessionCookie" string to look up the separate "RTSPClientSession" object that should have been used to handle
   // an earlier HTTP "GET" request:
   RTSPServer::RTSPClientSession* prevClientSession
@@ -1306,12 +1308,12 @@ Boolean RTSPServer::RTSPClientSession::handleHTTPCmd_POST(char const* sessionCoo
     fSessionIsActive = False; // triggers deletion of ourself
     return False;
   }
-
-  // Change the previous "RTSPClientSession" object's input socket to ours.  It will be used for subsequent requests:
-  prevClientSession->changeClientInputSocket(fClientInputSocket);
 #ifdef DEBUG
   fprintf(stderr, "Handled HTTP \"POST\" request (client input socket: %d)\n", fClientInputSocket);
 #endif
+
+  // Change the previous "RTSPClientSession" object's input socket to ours.  It will be used for subsequent requests:
+  prevClientSession->changeClientInputSocket(fClientInputSocket, extraData, extraDataSize);
   fClientInputSocket = fClientOutputSocket = -1; // so the socket doesn't get closed when we get deleted
   return True;
 }
@@ -1467,11 +1469,20 @@ RTSPServer::createNewClientSession(unsigned sessionId, int clientSocket, struct
 }
 
 void RTSPServer::RTSPClientSession
-::changeClientInputSocket(int newSocketNum) {
+::changeClientInputSocket(int newSocketNum, unsigned char const* extraData, unsigned extraDataSize) {
   envir().taskScheduler().turnOffBackgroundReadHandling(fClientInputSocket);
   fClientInputSocket = newSocketNum;
   envir().taskScheduler().turnOnBackgroundReadHandling(fClientInputSocket,
      (TaskScheduler::BackgroundHandlerProc*)&incomingRequestHandler, this);
+
+  // Also write any extra data to our buffer, and handle it:
+  if (extraDataSize > 0 && extraDataSize <= fRequestBufferBytesLeft/*sanity check; should always be true*/) {
+    unsigned char* ptr = &fRequestBuffer[fRequestBytesAlreadySeen];
+    for (unsigned i = 0; i < extraDataSize; ++i) {
+      ptr[i] = extraData[i];
+    }
+    handleRequestBytes(extraDataSize);
+  }
 }
 
 
